@@ -33,7 +33,7 @@ ArgSubstitution = namedtuple("ArgSubstitution", ["arg_name", "access_func", "key
 @dataclass
 class EvalEnv:
     """
-    Object to pass which parameter references are available for each node throughout walking the graph.
+    Object to keep state of which node in the graph is currently being walked.
     """
 
     parent: Optional[EvalEnv]
@@ -41,22 +41,9 @@ class EvalEnv:
     node_name: str
     process_graph_uid: str
     result: bool = False
-    parameters: set[str] = field(default_factory=set)
     result_references_to_walk: list[EvalEnv] = field(default_factory=list)
     callbacks_to_walk: dict[str, ProcessGraph] = field(default_factory=dict)
 
-    def search_for_parameter_env(self, arg_name: str) -> EvalEnv:
-        """
-        Recursively search for a parameter in a node's lineage. The most specific parameter (i.e. from the closest ancestor) is used.
-        """
-        if arg_name in self.parameters:
-            return self
-        if self.parent:
-            return self.parent.search_for_parameter_env(arg_name)
-        raise ProcessParameterMissing(
-            f"ProcessParameter {arg_name} missing for node {self.node_uid}."
-        )
-
     # This decorator makes this property not recompute each time it's called.
     @functools.cached_property
     def node_uid(self):
@@ -69,7 +56,6 @@ class EvalEnv:
         return f"""\n
         ---------------------------------------
         EVAL_ENV {self.node_uid}
-        parameters: {self.parameters}
         parent: {self.parent}
         ---------------------------------------
         """
@@ -78,10 +64,6 @@ class EvalEnv:
 UNRESOLVED_CALLBACK_VALUE = "__UNRESOLVED_CALLBACK__"
 
 
-class ProcessParameterMissing(Exception):
-    pass
-
-
 class OpenEOProcessGraph:
     def __init__(self, pg_data: dict):
         self.G = nx.DiGraph()
@@ -153,23 +135,10 @@ class OpenEOProcessGraph:
                 return
         raise Exception("Process graph has no return node!")
 
-    def _parse_argument(
-        self,
-        arg: ProcessArgument,
-        arg_name: str,
-        access_func: Callable,
-        real_origin_node: str = None,
-    ):
-
-        if isinstance(arg, ParameterReference):
-            # Search parent nodes for the referenced parameter.
-            # self._resolve_parameter_reference(
-            #     parameter_reference=arg, arg_name=arg_name, access_func=access_func
-            # )
-            pass
-
-        elif isinstance(arg, ResultReference):
-            # Only add a subnode for walking if it's in the same process grpah, otherwise you get infinite loops!
+    def _parse_argument(self, arg: any, arg_name: str, access_func: Callable):
+        if isinstance(arg, ResultReference):
+            # Finding a ResultReferences means that a new edge is required and the
+            # node specified in `from_node` has to be added to the nodes that potentially need walking.
             from_node_eval_env = EvalEnv(
                 parent=self._EVAL_ENV.parent,
                 node=arg.node,
@@ -177,9 +146,7 @@ class OpenEOProcessGraph:
                 process_graph_uid=self._EVAL_ENV.process_graph_uid,
             )
 
-            target_node = (
-                real_origin_node if real_origin_node else from_node_eval_env.node_uid
-            )
+            target_node = from_node_eval_env.node_uid
 
             self.G.add_edge(
                 self._EVAL_ENV.node_uid,
@@ -187,28 +154,20 @@ class OpenEOProcessGraph:
                 reference_type=PGEdgeType.ResultReference,
             )
 
-            if (
-                "arg_substitutions"
-                not in self.G.edges[self._EVAL_ENV.node_uid, target_node]
-            ):
-                self.G.edges[self._EVAL_ENV.node_uid, target_node][
-                    "arg_substitutions"
-                ] = []
+            edge_data = self.G.edges[self._EVAL_ENV.node_uid, target_node]
 
-            self.G.edges[self._EVAL_ENV.node_uid, target_node][
-                "arg_substitutions"
-            ].append(
+            if "arg_substitutions" not in edge_data:
+                edge_data["arg_substitutions"] = []
+
+            edge_data["arg_substitutions"].append(
                 ArgSubstitution(arg_name=arg_name, access_func=access_func, key=arg_name)
             )
 
-            if (
-                from_node_eval_env.process_graph_uid == self._EVAL_ENV.process_graph_uid
-                and not real_origin_node
-            ):
+            # Only add a subnode for walking if it's in the same process graph, otherwise you get infinite loops!
+            if from_node_eval_env.process_graph_uid == self._EVAL_ENV.process_graph_uid:
                 self._EVAL_ENV.result_references_to_walk.append(from_node_eval_env)
 
             access_func(new_value=arg, set_bool=True)
-            self._EVAL_ENV.parameters.add(arg_name)
 
         # dicts and list parameters can contain further result or parameter references, so have to parse these exhaustively.
         elif isinstance(arg, dict):
@@ -219,6 +178,7 @@ class OpenEOProcessGraph:
 
                 parsed_arg = parse_nested_parameter(v)
 
+                # This access func business is necessary to let the program "remember" how to access and thus update this reference later
                 sub_access_func = partial(
                     lambda key, access_func, new_value=None, set_bool=False: access_func()[
                         key
@@ -253,7 +213,6 @@ class OpenEOProcessGraph:
 
         else:
             access_func(new_value=arg, set_bool=True)
-            self._EVAL_ENV.parameters.add(arg_name)
 
     def _walk_node(self):
         """
@@ -343,7 +302,7 @@ class OpenEOProcessGraph:
         node: str,
         process_registry: dict,
         results_cache: Optional[dict] = None,
-        parameters: Optional[dict] = None,
+        named_parameters: Optional[dict] = None,
     ) -> Callable:
         """Recursively walk the graph from a given node to construct a callable that calls the process
         implementations of the given node and all its parent nodes and passes intermediate results between
@@ -352,8 +311,8 @@ class OpenEOProcessGraph:
         if results_cache is None:
             results_cache = {}
 
-        if parameters is None:
-            parameters = {}
+        if named_parameters is None:
+            named_parameters = {}
 
         node_with_data = self.G.nodes(data=True)[node]
         process_impl = process_registry[node_with_data["process_id"]]
@@ -363,38 +322,41 @@ class OpenEOProcessGraph:
 
         for _, source_node, data in self.G.out_edges(node, data=True):
             if data["reference_type"] == PGEdgeType.ResultReference:
-                parent_callables.append(
-                    self._map_node_to_callable(
-                        source_node,
-                        process_registry=process_registry,
-                        results_cache=results_cache,
-                        parameters=parameters,
-                    )
+                parent_callable = self._map_node_to_callable(
+                    source_node,
+                    process_registry=process_registry,
+                    results_cache=results_cache,
+                    named_parameters=named_parameters,
                 )
+                parent_callables.append(parent_callable)
             elif data["reference_type"] == PGEdgeType.Callback:
                 callback = self._map_node_to_callable(
                     source_node,
                     process_registry=process_registry,
                     results_cache=results_cache,
-                    parameters=parameters,
+                    named_parameters=named_parameters,
                 )
                 static_parameters[data["arg_name"]] = callback
 
         prebaked_process_impl = partial(
-            process_impl, parameters=parameters, **static_parameters
+            process_impl, named_parameters=named_parameters, **static_parameters
         )
 
-        def node_callable(parent_callables, **kwargs):
+        def node_callable(*args, parent_callables, named_parameters=None, **kwargs):
+            if parent_callables is None:
+                parent_callables = []
+
+            if named_parameters is None:
+                named_parameters = {}
+
             # The node needs to first call all its parents, so that results are prepopulated in the results_cache
             for func in parent_callables:
-                func(**kwargs)
+                func(*args, named_parameters=named_parameters, **kwargs)
 
             try:
                 # If this node has already been computed once, just grab that result from the results_cache instead of recomputing it.
                 return results_cache.__getitem__(node)
             except KeyError:
-                dynamic_parameters = {}
-
                 for _, source_node, data in self.G.out_edges(node, data=True):
                     if data["reference_type"] == PGEdgeType.ResultReference:
                         for arg_sub in data["arg_substitutions"]:
@@ -402,15 +364,13 @@ class OpenEOProcessGraph:
                                 new_value=results_cache[source_node], set_bool=True
                             )
 
-                        dynamic_parameters[arg_sub.arg_name] = self.G.nodes(data=True)[
-                            node
-                        ]["resolved_kwargs"].__getitem__(arg_sub.arg_name)
+                        kwargs[arg_sub.arg_name] = self.G.nodes(data=True)[node][
+                            "resolved_kwargs"
+                        ].__getitem__(arg_sub.arg_name)
 
-                # If we have no dynamic parameters, we need to resolve to the insitu xarray. I.e, get data into first subgraph nodes
-                if not dynamic_parameters:
-                    dynamic_parameters = kwargs
-
-                result = prebaked_process_impl(**dynamic_parameters)
+                result = prebaked_process_impl(
+                    *args, named_parameters=named_parameters, **kwargs
+                )
 
                 results_cache[node] = result
 
@@ -492,6 +452,11 @@ class OpenEOProcessGraph:
             for edge in self.G.edges
         ]
 
+        # To help with truncated labels
+        import matplotlib.pyplot as plt
+
+        plt.margins(x=0.2)
+
         nx.draw_circular(
             self.G,
             labels=nx.get_node_attributes(self.G, "node_name"),
